@@ -13,7 +13,8 @@
 """Abstractions to interact with service models."""
 from collections import defaultdict
 
-from botocore.utils import CachedProperty, instance_cache, hyphenize_service_id
+from botocore.utils import (CachedProperty, instance_cache,
+                            hyphenize_service_id)
 from botocore.compat import OrderedDict
 from botocore.exceptions import MissingServiceIdError
 from botocore.exceptions import UndefinedModelAttributeError
@@ -55,7 +56,7 @@ class Shape(object):
                         'jsonvalue', 'timestampFormat', 'hostLabel']
     METADATA_ATTRS = ['required', 'min', 'max', 'sensitive', 'enum',
                       'idempotencyToken', 'error', 'exception',
-                      'endpointdiscoveryid', 'retryable']
+                      'endpointdiscoveryid', 'retryable', 'document', 'union']
     MAP_TYPE = OrderedDict
 
     def __init__(self, shape_name, shape_model, shape_resolver=None):
@@ -137,6 +138,8 @@ class Shape(object):
             * sensitive
             * required
             * idempotencyToken
+            * document
+            * union
 
         :rtype: dict
         :return: Metadata about the shape.
@@ -175,7 +178,7 @@ class Shape(object):
 class StructureShape(Shape):
     @CachedProperty
     def members(self):
-        members = self._shape_model['members']
+        members = self._shape_model.get('members', self.MAP_TYPE())
         # The members dict looks like:
         #    'members': {
         #        'MemberName': {'shape': 'shapeName'},
@@ -194,6 +197,25 @@ class StructureShape(Shape):
                 return member_name
         return None
 
+    @CachedProperty
+    def error_code(self):
+        if not self.metadata.get('exception', False):
+            return None
+        error_metadata = self.metadata.get("error", {})
+        code = error_metadata.get("code")
+        if code:
+            return code
+        # Use the exception name if there is no explicit code modeled
+        return self.name
+
+    @CachedProperty
+    def is_document_type(self):
+        return self.metadata.get('document', False)
+
+    @CachedProperty
+    def is_tagged_union(self):
+        return self.metadata.get('union', False)
+
 
 class ListShape(Shape):
     @CachedProperty
@@ -258,6 +280,17 @@ class ServiceModel(object):
         return self._shape_resolver.get_shape_by_name(
             shape_name, member_traits)
 
+    def shape_for_error_code(self, error_code):
+        return self._error_code_cache.get(error_code, None)
+
+    @CachedProperty
+    def _error_code_cache(self):
+        error_code_cache = {}
+        for error_shape in self.error_shapes:
+            code = error_shape.error_code
+            error_code_cache[code] = error_shape
+        return error_code_cache
+
     def resolve_shape_ref(self, shape_ref):
         return self._shape_resolver.resolve_shape_ref(shape_ref)
 
@@ -265,6 +298,15 @@ class ServiceModel(object):
     def shape_names(self):
         return list(self._service_description.get('shapes', {}))
 
+    @CachedProperty
+    def error_shapes(self):
+        error_shapes = []
+        for shape_name in self.shape_names:
+            error_shape = self.shape_for(shape_name)
+            if error_shape.metadata.get('exception', False):
+                error_shapes.append(error_shape)
+        return error_shapes
+
     @instance_cache
     def operation_model(self, operation_name):
         try:
@@ -338,6 +380,15 @@ class ServiceModel(object):
             if model.is_endpoint_discovery_operation:
                 return model
 
+    @CachedProperty
+    def endpoint_discovery_required(self):
+        for operation in self.operation_names:
+            model = self.operation_model(operation)
+            if (model.endpoint_discovery is not None and
+                    model.endpoint_discovery.get('required')):
+                return True
+        return False
+
     def _get_metadata_property(self, name):
         try:
             return self.metadata[name]
@@ -488,6 +539,10 @@ class OperationModel(object):
     def endpoint(self):
         return self._operation_model.get('endpoint')
 
+    @CachedProperty
+    def http_checksum_required(self):
+        return self._operation_model.get('httpChecksumRequired', False)
+
     @CachedProperty
     def has_event_stream_input(self):
         return self.get_event_stream_input() is not None
@@ -694,7 +749,7 @@ class DenormalizedStructureBuilder(object):
         shape = self._build_initial_shape(model)
         shape['members'] = members
 
-        for name, member_model in model['members'].items():
+        for name, member_model in model.get('members', OrderedDict()).items():
             member_shape_name = self._get_shape_name(member_model)
             members[name] = {'shape': member_shape_name}
             self._build_model(member_model, shapes, member_shape_name)
